function [Problem,ObjValue] = PenaltyEval(Problem, x, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Subrotine PenaltyEval
%  This subrotine is called to evaluate the objective function
%
%  Input:
%    Problem - The problem structure
%    x - Real part of point to evaluate
%    varargin - Extra parameters for objective function
%
%  Output:
%    Problem - The problem structure updated
%    ObjValue - Objective function value. Returns +Inf for unfeasible
%       points
%
% Copyright (C) 2007 A. Ismael F. Vaz and L. N. Vicente.
%
% http://www.norg.uminho.pt/aivaz
% http://www.mat.uc.pt/~lnv
%
% This library is free software; you can redistribute it and/or
% modify it under the terms of the GNU Lesser General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.
%
% This library is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% Lesser General Public License for more details.
%
% You should have received a copy of the GNU Lesser General Public
% License along with this library; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%aivaz@dps.uminho.pt 06/12/2006

% Bound feasibility is enforced by the projection onto the bound feasible
% domain
% LB <= x' && x'<=UB
% Be paranoic

% There is no need to check for Problem.Vectorized, since we are assuming a
% matrix with NPoints for penalty function evaluation
[NPoints,Vars]=size(x);
ObjValue(1:NPoints)=+Inf;

% Check for indices of points that are bound feasible
FeasibleBoundsIdx=find(min(x'-repmat(Problem.LB,1,NPoints))>=0 & min(repmat(Problem.UB,1,NPoints)-x')>=0);

if Problem.mLinear==0
	FeasibleLinearIdx=FeasibleBoundsIdx;
else
    FeasibleLinearIdx=find(max(Problem.A*x(FeasibleBoundsIdx,:)'-repmat(Problem.b,1,length(FeasibleBoundsIdx)))<=0);
end

PointsEval=x(FeasibleLinearIdx,:)';
[Vars,NPoints]=size(PointsEval);

% Compute objective function value
try
    ObjValue(FeasibleLinearIdx)=feval(Problem.ObjFunction, PointsEval, varargin{:});
    % update counter
    Problem.Stats.ObjFunCounter=Problem.Stats.ObjFunCounter+NPoints;
catch
    error('pswarm:ObjectiveError', ...
        ['Cannot continue because user supplied objective function' ...
        ' failed with the following error:\n%s'], lasterr)
end

return;